package lab5;
import java.util.ArrayList;
import java.util.HashSet;

public class CNFSubClause
{
    private HashSet<Literal> literals = new HashSet<Literal>();

    HashSet<Literal> getLiterals() { return this.literals; }

    boolean isEmpty() { return this.literals.isEmpty(); }

    void print()
    {
        System.out.println("**************");
        for(Literal l: this.literals)
        {
            l.print();
        }
        System.out.println("**************");
    }

    /* Applies resolution on two CNFSubClauses
     * The resulting clause will contain all the literals of both CNFSubclauses
     * except the pair of literals that are a negation of each other.
     */
    static ArrayList<CNFSubClause> resolve(CNFSubClause C1, CNFSubClause C2)
    {
        ArrayList<CNFSubClause> newSubClause = new ArrayList<CNFSubClause>();

        //The iterator goes through all Literals of the first clause
        for(Literal l: C1.getLiterals())
        {
            Literal m = new Literal(l.getName(), !l.isNegated()); // negated l

            //If the second clause contains the negation of a Literal in the first clause
            if(C2.getLiterals().contains(m))
            {
                //We construct a new clause that contains all the literals of both CNFSubclauses...
                HashSet<Literal> C1exceptL = new HashSet<Literal>(C1.getLiterals());
                C1exceptL.remove(l); // remove l

                HashSet<Literal> C2exceptM = new HashSet<Literal>(C2.getLiterals());
                C2exceptM.remove(m); // remove m

                //Normally we have to remove duplicates of the same literal; the new clause must not contain the same literal more than once
                //But since we use HashSet only one copy of a literal will be contained anyway
                CNFSubClause result = new CNFSubClause();

                result.getLiterals().addAll(C1exceptL);
                result.getLiterals().addAll(C2exceptM);

                newSubClause.add(result);
            }

        }//The loop runs for all literals, producing a different new clause for each different pair of literals that negate each other

        return newSubClause;
    }

    @Override
    public boolean equals(Object obj)
    {
        CNFSubClause c = (CNFSubClause)obj;

        if(c.getLiterals().size() != this.getLiterals().size()) return false;

        for(Literal lit: c.getLiterals())
        {
            if(!this.getLiterals().contains(lit)) return false;
        }

        return true;
    }

    @Override
    public int hashCode()
    {
        int code = 0;
        for(Literal l: this.literals)
        {
            code += l.hashCode();
        }

        return code;
    }
}
